'use strict';

/**
 * Create a new HUD (heads-up display) instance.
 * @param {Game} game - The game in which the HUD is to be displayed
 */
function HUD(game) {
	this._game = game;
	
	/** {HTMLDivElement} The container for the HUD */
	this.topBar = document.createElement('div');
	this.topBar.id = 'game-hud';
	
	/** @private {HTMLElement} The container for the in-game health display */
	this._healthDisplay;
	/** @private {HTMLElement} The container for the in-game score display */
	this._scoreDisplay;
	/** @private {HTMLElement} The container for the in-game level counter */
	this._levelDisplay;
	
	/** {HTMLElement} The container for the level up message */
	this.levelUpMessage = document.createElement('div');
	this.levelUpMessage.id = 'game-level-up-message';
	this.levelUpMessage.innerText = 'Level Up!';
	
	/** {HTMLButtonElement} The button that toggles sound muting */
	this.soundToggleButton;
	/** @private {HTMLButtonElement} The button that pauses the game */
	this._pauseButton;

	this._health = 3;
	
	this.loadPromise =
		Utils.fetch(this.HTML_PATH)
			.then(this._setUp.bind(this))
			.catch(function (err) {
				console.error(err);
			});
}

// Define constants.
/** @constant {String} The path to the partial with the HUD HTML */
HUD.prototype.HTML_PATH = 'shared/partials/hud.html';
/** @constant {Number} The duration of the HUD flash animation, in milliseconds */
HUD.prototype.FLASH_ANIMATION_DURATION = 400;
/** @constant {Number} The duration of the level up message animation, in milliseconds */
HUD.prototype.LEVEL_UP_ANIMATION_DURATION = 1500;

/**
 * @private
 * Set up the HUD DOM once its partial has loaded.
 * @param {String} partialHTML - The HTML of the loaded partial
 */
HUD.prototype._setUp = function (partialHTML) {
	this.topBar.innerHTML = partialHTML;
	
	this._healthDisplay = this.topBar.querySelector('#game-health');
	this._scoreDisplay = this.topBar.querySelector('#game-score');
	this._levelDisplay = this.topBar.querySelector('#game-level');
	
	this._soundToggleButton = this.topBar.querySelector('#game-sound-button');
	this._soundToggleButton.onclick = this._toggleSound.bind(this);
	
	this._pauseButton = this.topBar.querySelector('#game-pause-button');
	this._pauseButton.onclick = this._game.pause.bind(this._game);
};

/**
 * @private
 * Toggle sound effects and music playing.
 */
HUD.prototype._toggleSound = function () {
	var ICON_FOLDER = 'shared/images/game/',
		SOUND_OFF_ICON = 'icons.svg#sound-off',
		SOUND_ON_ICON = 'icons.svg#sound-on';
	this._game.muted = !this._game.muted;
	this._soundToggleButton.setAttribute('aria-pressed', this._game.muted);
	this._soundToggleButton.innerHTML = '<svg>' +
		'<title>' + (this._game.muted ? 'Muted' : 'Unmuted') + '</title>' +
		'<use href="' + (ICON_FOLDER + (this._game.muted ? SOUND_OFF_ICON : SOUND_ON_ICON)) + '"></use>' +
	'</svg>';
};

/**
 * Toggle the state of the pause button.
 * @param {Boolean} [forceState] - The state to force
 */
HUD.prototype.togglePause = function (forceState) {
	var paused = true;
	
	if (typeof(forceState) !== 'undefined') {
		paused = forceState;
	} else {
		paused = (this._hud.pauseButton.getAttribute('aria-pressed') === 'true');
	}
	
	this._pauseButton.onclick = (paused ? this._game.resume.bind(this._game) : this._game.pause.bind(this._game));
	this._pauseButton.setAttribute('aria-pressed', paused);
	this._pauseButton.innerHTML =
		'<svg><title>Play</title><use href="shared/images/game/icons.svg#' +
		(paused ? 'play' : 'pause') +
		'"></use></svg>';
	this._game.app.view.classList[(paused ? 'add' : 'remove')]('blurred');
};

/**
 * Update the health display to a new amount.
 * @param {Number} newHealth - The new health amount
 * @param {Boolean} [noFlash] - Whether to disable the flash animation
 */
HUD.prototype.updateHealth = function (newHealth, noFlash) {
	// Update the health indicators.
	Array.from(this._healthDisplay.getElementsByTagName('svg')).forEach(function (heart, i) {
		heart.classList[(newHealth > i) ? 'add' : 'remove']('full');
	});
	this._healthDisplay.setAttribute('aria-valuenow', newHealth);
	this._healthDisplay.setAttribute('aria-valuetext', newHealth + ' health');
	
	// Flash the health display.
	if (!noFlash) {
		this._healthDisplay.style.animationName = 'quick-flash';
		setTimeout((function () {
			this._healthDisplay.style.animationName = 'none';
		}).bind(this), this.FLASH_ANIMATION_DURATION);
	}
};

/**
 * Update the score display to a new value.
 * @param {Number} newScore - The new score value
 * @param {Boolean} [noFlash] - Whether to disable the flash animation
 */
HUD.prototype.updateScore = function (newScore, noFlash) {
	// Update the score display.
	this._scoreDisplay.innerText = 'Score: ' + newScore;
	
	// Flash the score display.
	if (!noFlash) {
		this._scoreDisplay.style.animationName = 'quick-flash';
		setTimeout((function () {
			this._scoreDisplay.style.animationName = 'none'
		}).bind(this), this.FLASH_ANIMATION_DURATION);
	}
};

/**
 * Update the level counter to a new value.
 * @param {Number} newLevel - The new level number
 * @param {Boolean} [noFlash] - Whether to disable the flash animation
 */
HUD.prototype.updateLevel = function (newLevel, noFlash) {
	// Update the score display.
	this._levelDisplay.innerText = 'Level: ' + newLevel;
	if (!this._game.muted) {
		this._game.levelUpSound.currentTime = 0;
		this._game.levelUpSound.play();
	}
	// Flash the score display.
	if (!noFlash) {
		this._levelDisplay.style.animationName = 'quick-flash';
		setTimeout((function () {
			this._levelDisplay.style.animationName = 'none'
		}).bind(this), this.FLASH_ANIMATION_DURATION);
	}
};

/**
 * Show the animated “Level Up” message.
 */
HUD.prototype.showLevelUpMessage = function () {
	this.levelUpMessage.classList.add('active');
	setTimeout((function () {
		this.levelUpMessage.classList.remove('active');
	}).bind(this), this.LEVEL_UP_ANIMATION_DURATION);
};
